/********************************************************************************
** Form generated from reading UI file 'edittypewidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EDITTYPEWIDGET_H
#define UI_EDITTYPEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "KIconButton"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_EditTypeWidget
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *label;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QLabel *label_2_3;
    KLineEdit *editName;
    QHBoxLayout *hboxLayout2;
    QLabel *label_2_2_2;
    KLineEdit *editDescription;
    QVBoxLayout *vboxLayout2;
    KIconButton *typeIcon;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *EditTypeWidget)
    {
        if (EditTypeWidget->objectName().isEmpty())
            EditTypeWidget->setObjectName("EditTypeWidget");
        EditTypeWidget->resize(478, 114);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(EditTypeWidget->sizePolicy().hasHeightForWidth());
        EditTypeWidget->setSizePolicy(sizePolicy);
        EditTypeWidget->setMinimumSize(QSize(200, 100));
        vboxLayout = new QVBoxLayout(EditTypeWidget);
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        label = new QLabel(EditTypeWidget);
        label->setObjectName("label");
        label->setTextFormat(Qt::PlainText);
        label->setWordWrap(true);

        vboxLayout->addWidget(label);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        hboxLayout1 = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout1->setObjectName("hboxLayout1");
        label_2_3 = new QLabel(EditTypeWidget);
        label_2_3->setObjectName("label_2_3");

        hboxLayout1->addWidget(label_2_3);

        editName = new KLineEdit(EditTypeWidget);
        editName->setObjectName("editName");

        hboxLayout1->addWidget(editName);


        vboxLayout1->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout2->setSpacing(6);
#endif
        hboxLayout2->setContentsMargins(0, 0, 0, 0);
        hboxLayout2->setObjectName("hboxLayout2");
        label_2_2_2 = new QLabel(EditTypeWidget);
        label_2_2_2->setObjectName("label_2_2_2");

        hboxLayout2->addWidget(label_2_2_2);

        editDescription = new KLineEdit(EditTypeWidget);
        editDescription->setObjectName("editDescription");

        hboxLayout2->addWidget(editDescription);


        vboxLayout1->addLayout(hboxLayout2);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
#ifndef UI_Q_OS_MACOS
        vboxLayout2->setSpacing(6);
#endif
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        vboxLayout2->setObjectName("vboxLayout2");
        typeIcon = new KIconButton(EditTypeWidget);
        typeIcon->setObjectName("typeIcon");
        typeIcon->setIconSize(22);

        vboxLayout2->addWidget(typeIcon);

        spacerItem = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout2->addItem(spacerItem);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        spacerItem1 = new QSpacerItem(20, 8, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Maximum);

        vboxLayout->addItem(spacerItem1);


        retranslateUi(EditTypeWidget);

        QMetaObject::connectSlotsByName(EditTypeWidget);
    } // setupUi

    void retranslateUi(QWidget *EditTypeWidget)
    {
        label->setText(tr2i18n("Here you can modify the name, the description and the icon of this macro type.", nullptr));
        label_2_3->setText(tr2i18n("Name:", nullptr));
        label_2_2_2->setText(tr2i18n("Description:", nullptr));
        (void)EditTypeWidget;
    } // retranslateUi

};

namespace Ui {
    class EditTypeWidget: public Ui_EditTypeWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // EDITTYPEWIDGET_H

