/********************************************************************************
** Form generated from reading UI file 'latexexporteroptionswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LATEXEXPORTEROPTIONSWIDGET_H
#define UI_LATEXEXPORTEROPTIONSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_LatexExporterOptionsWidget
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *psTricksRadioButton;
    QRadioButton *tikzRadioButton;
    QRadioButton *asyRadioButton;
    QGroupBox *groupBox_3;
    QHBoxLayout *horizontalLayout;
    QRadioButton *pictureRadioButton;
    QRadioButton *documentRadioButton;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QCheckBox *showGridCheckBox;
    QCheckBox *showFrameCheckBox;
    QCheckBox *showAxesCheckBox;

    void setupUi(QWidget *LatexExporterOptionsWidget)
    {
        if (LatexExporterOptionsWidget->objectName().isEmpty())
            LatexExporterOptionsWidget->setObjectName("LatexExporterOptionsWidget");
        LatexExporterOptionsWidget->resize(417, 375);
        vboxLayout = new QVBoxLayout(LatexExporterOptionsWidget);
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        groupBox_2 = new QGroupBox(LatexExporterOptionsWidget);
        groupBox_2->setObjectName("groupBox_2");
        horizontalLayout_2 = new QHBoxLayout(groupBox_2);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        psTricksRadioButton = new QRadioButton(groupBox_2);
        psTricksRadioButton->setObjectName("psTricksRadioButton");

        horizontalLayout_2->addWidget(psTricksRadioButton);

        tikzRadioButton = new QRadioButton(groupBox_2);
        tikzRadioButton->setObjectName("tikzRadioButton");
        tikzRadioButton->setChecked(true);

        horizontalLayout_2->addWidget(tikzRadioButton);

        asyRadioButton = new QRadioButton(groupBox_2);
        asyRadioButton->setObjectName("asyRadioButton");

        horizontalLayout_2->addWidget(asyRadioButton);


        vboxLayout->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(LatexExporterOptionsWidget);
        groupBox_3->setObjectName("groupBox_3");
        horizontalLayout = new QHBoxLayout(groupBox_3);
        horizontalLayout->setObjectName("horizontalLayout");
        pictureRadioButton = new QRadioButton(groupBox_3);
        pictureRadioButton->setObjectName("pictureRadioButton");
        pictureRadioButton->setChecked(true);

        horizontalLayout->addWidget(pictureRadioButton);

        documentRadioButton = new QRadioButton(groupBox_3);
        documentRadioButton->setObjectName("documentRadioButton");

        horizontalLayout->addWidget(documentRadioButton);


        vboxLayout->addWidget(groupBox_3);

        groupBox = new QGroupBox(LatexExporterOptionsWidget);
        groupBox->setObjectName("groupBox");
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName("gridLayout");
        showGridCheckBox = new QCheckBox(groupBox);
        showGridCheckBox->setObjectName("showGridCheckBox");

        gridLayout->addWidget(showGridCheckBox, 0, 0, 1, 1);

        showFrameCheckBox = new QCheckBox(groupBox);
        showFrameCheckBox->setObjectName("showFrameCheckBox");

        gridLayout->addWidget(showFrameCheckBox, 1, 0, 1, 1);

        showAxesCheckBox = new QCheckBox(groupBox);
        showAxesCheckBox->setObjectName("showAxesCheckBox");

        gridLayout->addWidget(showAxesCheckBox, 0, 1, 1, 1);


        vboxLayout->addWidget(groupBox);


        retranslateUi(LatexExporterOptionsWidget);

        QMetaObject::connectSlotsByName(LatexExporterOptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *LatexExporterOptionsWidget)
    {
        groupBox_2->setTitle(tr2i18n("Output format", nullptr));
        psTricksRadioButton->setText(tr2i18n("PSTricks", nullptr));
        tikzRadioButton->setText(tr2i18n("Tikz/PGF", nullptr));
        asyRadioButton->setText(tr2i18n("Asymptote", nullptr));
        groupBox_3->setTitle(tr2i18n("Document options", nullptr));
        pictureRadioButton->setText(tr2i18n("Picture only", nullptr));
        documentRadioButton->setText(tr2i18n("Standalone document", nullptr));
        groupBox->setTitle(tr2i18n("Options", nullptr));
        showGridCheckBox->setText(tr2i18n("Show Grid", nullptr));
        showFrameCheckBox->setText(tr2i18n("Show Extra Frame", nullptr));
        showAxesCheckBox->setText(tr2i18n("Show Axes", nullptr));
        (void)LatexExporterOptionsWidget;
    } // retranslateUi

};

namespace Ui {
    class LatexExporterOptionsWidget: public Ui_LatexExporterOptionsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // LATEXEXPORTEROPTIONSWIDGET_H

