/********************************************************************************
** Form generated from reading UI file 'imageexporteroptionswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEEXPORTEROPTIONSWIDGET_H
#define UI_IMAGEEXPORTEROPTIONSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ImageExporterOptionsWidget
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QCheckBox *keepAspectRatio;
    QDoubleSpinBox *HeightInput;
    QLabel *label_2;
    QVBoxLayout *vboxLayout1;
    QSpacerItem *spacerItem;
    QComboBox *comboUnit;
    QSpacerItem *spacerItem1;
    QDoubleSpinBox *WidthInput;
    QLabel *label;
    QGroupBox *groupBox_2;
    QVBoxLayout *vboxLayout2;
    QGridLayout *gridLayout1;
    QCheckBox *showAxesCheckBox;
    QCheckBox *showGridCheckBox;
    QSpacerItem *spacerItem2;

    void setupUi(QWidget *ImageExporterOptionsWidget)
    {
        if (ImageExporterOptionsWidget->objectName().isEmpty())
            ImageExporterOptionsWidget->setObjectName("ImageExporterOptionsWidget");
        ImageExporterOptionsWidget->resize(401, 198);
        vboxLayout = new QVBoxLayout(ImageExporterOptionsWidget);
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(ImageExporterOptionsWidget);
        groupBox->setObjectName("groupBox");
        groupBox->setCheckable(false);
        gridLayout = new QGridLayout(groupBox);
#ifndef UI_Q_OS_MACOS
        gridLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        gridLayout->setContentsMargins(9, 9, 9, 9);
#endif
        gridLayout->setObjectName("gridLayout");
        keepAspectRatio = new QCheckBox(groupBox);
        keepAspectRatio->setObjectName("keepAspectRatio");

        gridLayout->addWidget(keepAspectRatio, 2, 0, 1, 3);

        HeightInput = new QDoubleSpinBox(groupBox);
        HeightInput->setObjectName("HeightInput");
        HeightInput->setDecimals(0);
        HeightInput->setMaximum(2000.000000000000000);

        gridLayout->addWidget(HeightInput, 1, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        vboxLayout1 = new QVBoxLayout();
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem);

        comboUnit = new QComboBox(groupBox);
        comboUnit->setObjectName("comboUnit");

        vboxLayout1->addWidget(comboUnit);

        spacerItem1 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacerItem1);


        gridLayout->addLayout(vboxLayout1, 0, 2, 2, 1);

        WidthInput = new QDoubleSpinBox(groupBox);
        WidthInput->setObjectName("WidthInput");
        WidthInput->setDecimals(0);
        WidthInput->setMaximum(2000.000000000000000);

        gridLayout->addWidget(WidthInput, 0, 1, 1, 1);

        label = new QLabel(groupBox);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);


        vboxLayout->addWidget(groupBox);

        groupBox_2 = new QGroupBox(ImageExporterOptionsWidget);
        groupBox_2->setObjectName("groupBox_2");
        vboxLayout2 = new QVBoxLayout(groupBox_2);
#ifndef UI_Q_OS_MACOS
        vboxLayout2->setSpacing(6);
#endif
        vboxLayout2->setContentsMargins(8, 8, 8, 8);
        vboxLayout2->setObjectName("vboxLayout2");
        gridLayout1 = new QGridLayout();
#ifndef UI_Q_OS_MACOS
        gridLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        gridLayout1->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout1->setObjectName("gridLayout1");
        showAxesCheckBox = new QCheckBox(groupBox_2);
        showAxesCheckBox->setObjectName("showAxesCheckBox");

        gridLayout1->addWidget(showAxesCheckBox, 0, 1, 1, 1);

        showGridCheckBox = new QCheckBox(groupBox_2);
        showGridCheckBox->setObjectName("showGridCheckBox");

        gridLayout1->addWidget(showGridCheckBox, 0, 0, 1, 1);


        vboxLayout2->addLayout(gridLayout1);


        vboxLayout->addWidget(groupBox_2);

        spacerItem2 = new QSpacerItem(383, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem2);


        retranslateUi(ImageExporterOptionsWidget);

        QMetaObject::connectSlotsByName(ImageExporterOptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *ImageExporterOptionsWidget)
    {
        groupBox->setTitle(tr2i18n("Resolution", nullptr));
        keepAspectRatio->setText(tr2i18n("&Keep Aspect Ratio", nullptr));
        label_2->setText(tr2i18n("Height:", nullptr));
        label->setText(tr2i18n("Width:", nullptr));
        groupBox_2->setTitle(tr2i18n("Options", nullptr));
        showAxesCheckBox->setText(tr2i18n("Show Axes", nullptr));
        showGridCheckBox->setText(tr2i18n("Show Grid", nullptr));
        (void)ImageExporterOptionsWidget;
    } // retranslateUi

};

namespace Ui {
    class ImageExporterOptionsWidget: public Ui_ImageExporterOptionsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // IMAGEEXPORTEROPTIONSWIDGET_H

