/********************************************************************************
** Form generated from reading UI file 'typeswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TYPESWIDGET_H
#define UI_TYPESWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_TypesWidget
{
public:
    QVBoxLayout *vboxLayout;
    QTreeView *typeList;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QPushButton *buttonEdit;
    QPushButton *buttonRemove;
    QSpacerItem *spacerItem;
    QVBoxLayout *vboxLayout2;
    QPushButton *buttonExport;
    QPushButton *buttonImport;

    void setupUi(QWidget *TypesWidget)
    {
        if (TypesWidget->objectName().isEmpty())
            TypesWidget->setObjectName("TypesWidget");
        TypesWidget->resize(464, 253);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(TypesWidget->sizePolicy().hasHeightForWidth());
        TypesWidget->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(TypesWidget);
        vboxLayout->setSpacing(6);
        vboxLayout->setObjectName("vboxLayout");
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        typeList = new QTreeView(TypesWidget);
        typeList->setObjectName("typeList");
        typeList->setAlternatingRowColors(true);
        typeList->setSelectionMode(QAbstractItemView::ExtendedSelection);
        typeList->setIconSize(QSize(22, 22));
        typeList->setRootIsDecorated(false);
        typeList->setAllColumnsShowFocus(true);

        vboxLayout->addWidget(typeList);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setSpacing(6);
        vboxLayout1->setObjectName("vboxLayout1");
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        buttonEdit = new QPushButton(TypesWidget);
        buttonEdit->setObjectName("buttonEdit");

        vboxLayout1->addWidget(buttonEdit);

        buttonRemove = new QPushButton(TypesWidget);
        buttonRemove->setObjectName("buttonRemove");

        vboxLayout1->addWidget(buttonRemove);


        hboxLayout->addLayout(vboxLayout1);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setSpacing(6);
        vboxLayout2->setObjectName("vboxLayout2");
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        buttonExport = new QPushButton(TypesWidget);
        buttonExport->setObjectName("buttonExport");

        vboxLayout2->addWidget(buttonExport);

        buttonImport = new QPushButton(TypesWidget);
        buttonImport->setObjectName("buttonImport");

        vboxLayout2->addWidget(buttonImport);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(TypesWidget);

        QMetaObject::connectSlotsByName(TypesWidget);
    } // setupUi

    void retranslateUi(QWidget *TypesWidget)
    {
#if QT_CONFIG(tooltip)
        typeList->setToolTip(tr2i18n("Select types here...", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        typeList->setWhatsThis(tr2i18n("This is a list of the current macro types... You can select, edit, delete, export and import them...", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(whatsthis)
        buttonEdit->setWhatsThis(tr2i18n("Edit the selected type.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonEdit->setText(tr2i18n("&Edit...", nullptr));
#if QT_CONFIG(whatsthis)
        buttonRemove->setWhatsThis(tr2i18n("Delete all the selected types in the list.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonRemove->setText(tr2i18n("&Delete", nullptr));
#if QT_CONFIG(whatsthis)
        buttonExport->setWhatsThis(tr2i18n("Export all the selected types to a file.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonExport->setText(tr2i18n("E&xport...", nullptr));
#if QT_CONFIG(whatsthis)
        buttonImport->setWhatsThis(tr2i18n("Import macros that are contained in one or more files.", nullptr));
#endif // QT_CONFIG(whatsthis)
        buttonImport->setText(tr2i18n("&Import...", nullptr));
        (void)TypesWidget;
    } // retranslateUi

};

namespace Ui {
    class TypesWidget: public Ui_TypesWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // TYPESWIDGET_H

