/********************************************************************************
** Form generated from reading UI file 'kigcoordinateprecisiondialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KIGCOORDINATEPRECISIONDIALOG_H
#define UI_KIGCOORDINATEPRECISIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_KigCoordinatePrecisionDialog
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *m_defaultCheckBox;
    QLabel *m_precisionLabel;
    QSpinBox *m_precisionSpinBox;

    void setupUi(QWidget *KigCoordinatePrecisionDialog)
    {
        if (KigCoordinatePrecisionDialog->objectName().isEmpty())
            KigCoordinatePrecisionDialog->setObjectName("KigCoordinatePrecisionDialog");
        verticalLayout = new QVBoxLayout(KigCoordinatePrecisionDialog);
        verticalLayout->setObjectName("verticalLayout");
        m_defaultCheckBox = new QCheckBox(KigCoordinatePrecisionDialog);
        m_defaultCheckBox->setObjectName("m_defaultCheckBox");

        verticalLayout->addWidget(m_defaultCheckBox);

        m_precisionLabel = new QLabel(KigCoordinatePrecisionDialog);
        m_precisionLabel->setObjectName("m_precisionLabel");

        verticalLayout->addWidget(m_precisionLabel);

        m_precisionSpinBox = new QSpinBox(KigCoordinatePrecisionDialog);
        m_precisionSpinBox->setObjectName("m_precisionSpinBox");
        m_precisionSpinBox->setMaximum(15);

        verticalLayout->addWidget(m_precisionSpinBox);

#if QT_CONFIG(shortcut)
        m_precisionLabel->setBuddy(m_precisionSpinBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(KigCoordinatePrecisionDialog);

        QMetaObject::connectSlotsByName(KigCoordinatePrecisionDialog);
    } // setupUi

    void retranslateUi(QWidget *KigCoordinatePrecisionDialog)
    {
        m_defaultCheckBox->setText(tr2i18n("&Use default coordinate precision", nullptr));
        m_precisionLabel->setText(tr2i18n("&Specify coordinate precision:", nullptr));
        (void)KigCoordinatePrecisionDialog;
    } // retranslateUi

};

namespace Ui {
    class KigCoordinatePrecisionDialog: public Ui_KigCoordinatePrecisionDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KIGCOORDINATEPRECISIONDIALOG_H

