/****************************************************************************
** Meta object code from reading C++ file 'kig_part.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kig-26.07.80/kig/kig_part.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kig_part.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN25SetCoordinateSystemActionE_t {};
} // unnamed namespace

template <> constexpr inline auto SetCoordinateSystemAction::qt_create_metaobjectdata<qt_meta_tag_ZN25SetCoordinateSystemActionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "SetCoordinateSystemAction",
        "slotActivated",
        "",
        "index"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'slotActivated'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SetCoordinateSystemAction, qt_meta_tag_ZN25SetCoordinateSystemActionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject SetCoordinateSystemAction::staticMetaObject = { {
    QMetaObject::SuperData::link<KSelectAction::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25SetCoordinateSystemActionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25SetCoordinateSystemActionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25SetCoordinateSystemActionE_t>.metaTypes,
    nullptr
} };

void SetCoordinateSystemAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SetCoordinateSystemAction *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->slotActivated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *SetCoordinateSystemAction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SetCoordinateSystemAction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25SetCoordinateSystemActionE_t>.strings))
        return static_cast<void*>(this);
    return KSelectAction::qt_metacast(_clname);
}

int SetCoordinateSystemAction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KSelectAction::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN7KigPartE_t {};
} // unnamed namespace

template <> constexpr inline auto KigPart::qt_create_metaobjectdata<qt_meta_tag_ZN7KigPartE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KigPart",
        "recenterScreen",
        "",
        "fileSaveAs",
        "fileSave",
        "filePrint",
        "filePrintPreview",
        "slotSelectAll",
        "slotDeselectAll",
        "slotInvertSelection",
        "unplugActionLists",
        "plugActionLists",
        "deleteObjects",
        "cancelConstruction",
        "repeatLastConstruction",
        "showHidden",
        "newMacro",
        "editTypes",
        "browseHistory",
        "toggleGrid",
        "toggleAxes",
        "toggleNightVision",
        "setHistoryClean",
        "setCoordinatePrecision"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'recenterScreen'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSaveAs'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fileSave'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filePrint'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'filePrintPreview'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSelectAll'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotDeselectAll'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotInvertSelection'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'unplugActionLists'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'plugActionLists'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteObjects'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'cancelConstruction'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'repeatLastConstruction'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showHidden'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newMacro'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editTypes'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'browseHistory'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleGrid'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleAxes'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toggleNightVision'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setHistoryClean'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'setCoordinatePrecision'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KigPart, qt_meta_tag_ZN7KigPartE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KigPart::staticMetaObject = { {
    QMetaObject::SuperData::link<KParts::ReadWritePart::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KigPartE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KigPartE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7KigPartE_t>.metaTypes,
    nullptr
} };

void KigPart::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KigPart *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->recenterScreen(); break;
        case 1: _t->fileSaveAs(); break;
        case 2: _t->fileSave(); break;
        case 3: _t->filePrint(); break;
        case 4: _t->filePrintPreview(); break;
        case 5: _t->slotSelectAll(); break;
        case 6: _t->slotDeselectAll(); break;
        case 7: _t->slotInvertSelection(); break;
        case 8: _t->unplugActionLists(); break;
        case 9: _t->plugActionLists(); break;
        case 10: _t->deleteObjects(); break;
        case 11: _t->cancelConstruction(); break;
        case 12: _t->repeatLastConstruction(); break;
        case 13: _t->showHidden(); break;
        case 14: _t->newMacro(); break;
        case 15: _t->editTypes(); break;
        case 16: _t->browseHistory(); break;
        case 17: _t->toggleGrid(); break;
        case 18: _t->toggleAxes(); break;
        case 19: _t->toggleNightVision(); break;
        case 20: _t->setHistoryClean((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->setCoordinatePrecision(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KigPart::*)()>(_a, &KigPart::recenterScreen, 0))
            return;
    }
}

const QMetaObject *KigPart::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KigPart::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7KigPartE_t>.strings))
        return static_cast<void*>(this);
    return KParts::ReadWritePart::qt_metacast(_clname);
}

int KigPart::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KParts::ReadWritePart::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void KigPart::recenterScreen()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
