/********************************************************************************
** Form generated from reading UI file 'svgexporteroptionswidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SVGEXPORTEROPTIONSWIDGET_H
#define UI_SVGEXPORTEROPTIONSWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SVGExporterOptionsWidget
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox_2;
    QVBoxLayout *vboxLayout1;
    QGridLayout *gridLayout;
    QCheckBox *showAxesCheckBox;
    QCheckBox *showGridCheckBox;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *SVGExporterOptionsWidget)
    {
        if (SVGExporterOptionsWidget->objectName().isEmpty())
            SVGExporterOptionsWidget->setObjectName("SVGExporterOptionsWidget");
        SVGExporterOptionsWidget->resize(400, 90);
        vboxLayout = new QVBoxLayout(SVGExporterOptionsWidget);
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(8, 8, 8, 8);
        vboxLayout->setObjectName("vboxLayout");
        groupBox_2 = new QGroupBox(SVGExporterOptionsWidget);
        groupBox_2->setObjectName("groupBox_2");
        vboxLayout1 = new QVBoxLayout(groupBox_2);
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setSpacing(6);
#endif
        vboxLayout1->setContentsMargins(8, 8, 8, 8);
        vboxLayout1->setObjectName("vboxLayout1");
        gridLayout = new QGridLayout();
#ifndef UI_Q_OS_MACOS
        gridLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        gridLayout->setContentsMargins(0, 0, 0, 0);
#endif
        gridLayout->setObjectName("gridLayout");
        showAxesCheckBox = new QCheckBox(groupBox_2);
        showAxesCheckBox->setObjectName("showAxesCheckBox");

        gridLayout->addWidget(showAxesCheckBox, 0, 1, 1, 1);

        showGridCheckBox = new QCheckBox(groupBox_2);
        showGridCheckBox->setObjectName("showGridCheckBox");

        gridLayout->addWidget(showGridCheckBox, 0, 0, 1, 1);


        vboxLayout1->addLayout(gridLayout);


        vboxLayout->addWidget(groupBox_2);

        spacerItem = new QSpacerItem(20, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);


        retranslateUi(SVGExporterOptionsWidget);

        QMetaObject::connectSlotsByName(SVGExporterOptionsWidget);
    } // setupUi

    void retranslateUi(QWidget *SVGExporterOptionsWidget)
    {
        groupBox_2->setTitle(tr2i18n("Options", nullptr));
        showAxesCheckBox->setText(tr2i18n("Show Axes", nullptr));
        showGridCheckBox->setText(tr2i18n("Show Grid", nullptr));
        (void)SVGExporterOptionsWidget;
    } // retranslateUi

};

namespace Ui {
    class SVGExporterOptionsWidget: public Ui_SVGExporterOptionsWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SVGEXPORTEROPTIONSWIDGET_H

