/********************************************************************************
** Form generated from reading UI file 'historywidget.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HISTORYWIDGET_H
#define UI_HISTORYWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_HistoryWidget
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *label;
    QTextEdit *description;
    QHBoxLayout *hboxLayout;
    QPushButton *buttonFirst;
    QPushButton *buttonBack;
    KLineEdit *editStep;
    QLabel *label_2;
    QLabel *labelSteps;
    QPushButton *buttonNext;
    QPushButton *buttonLast;

    void setupUi(QWidget *HistoryWidget)
    {
        if (HistoryWidget->objectName().isEmpty())
            HistoryWidget->setObjectName("HistoryWidget");
        HistoryWidget->resize(433, 155);
        vboxLayout = new QVBoxLayout(HistoryWidget);
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName("vboxLayout");
        label = new QLabel(HistoryWidget);
        label->setObjectName("label");
        label->setWordWrap(true);

        vboxLayout->addWidget(label);

        description = new QTextEdit(HistoryWidget);
        description->setObjectName("description");
        description->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        description->setUndoRedoEnabled(false);
        description->setReadOnly(true);

        vboxLayout->addWidget(description);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        buttonFirst = new QPushButton(HistoryWidget);
        buttonFirst->setObjectName("buttonFirst");
        buttonFirst->setFlat(false);

        hboxLayout->addWidget(buttonFirst);

        buttonBack = new QPushButton(HistoryWidget);
        buttonBack->setObjectName("buttonBack");
        buttonBack->setFlat(false);

        hboxLayout->addWidget(buttonBack);

        editStep = new KLineEdit(HistoryWidget);
        editStep->setObjectName("editStep");

        hboxLayout->addWidget(editStep);

        label_2 = new QLabel(HistoryWidget);
        label_2->setObjectName("label_2");

        hboxLayout->addWidget(label_2);

        labelSteps = new QLabel(HistoryWidget);
        labelSteps->setObjectName("labelSteps");

        hboxLayout->addWidget(labelSteps);

        buttonNext = new QPushButton(HistoryWidget);
        buttonNext->setObjectName("buttonNext");
        buttonNext->setFlat(false);

        hboxLayout->addWidget(buttonNext);

        buttonLast = new QPushButton(HistoryWidget);
        buttonLast->setObjectName("buttonLast");
        buttonLast->setFlat(false);

        hboxLayout->addWidget(buttonLast);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(HistoryWidget);

        QMetaObject::connectSlotsByName(HistoryWidget);
    } // setupUi

    void retranslateUi(QWidget *HistoryWidget)
    {
        label->setText(tr2i18n("Description of the current step:", nullptr));
#if QT_CONFIG(tooltip)
        buttonFirst->setToolTip(tr2i18n("First step", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonFirst->setText(QString());
#if QT_CONFIG(tooltip)
        buttonBack->setToolTip(tr2i18n("One step back", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonBack->setText(QString());
        label_2->setText(tr2i18n("/", nullptr));
        labelSteps->setText(QString());
#if QT_CONFIG(tooltip)
        buttonNext->setToolTip(tr2i18n("One step forward", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonNext->setText(QString());
#if QT_CONFIG(tooltip)
        buttonLast->setToolTip(tr2i18n("Last step", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonLast->setText(QString());
        (void)HistoryWidget;
    } // retranslateUi

};

namespace Ui {
    class HistoryWidget: public Ui_HistoryWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // HISTORYWIDGET_H

